#include "FreeRTOS.h"
#include "efm32_dma.h"
#include "efm32_cmu.h"
#include "dma.h"
#include "leuart.h"
#include "dmactrl.h"
#include "gpio.h"
#include "semphr.h"

xSemaphoreHandle xAccelSemaphore;
char string[200];

static void accelDMAConfig( void );

static void leuartDMAConfig(void);

static void accelDMAConfig(void)
{
  DMA_Init_TypeDef dmaInit;
  DMA_CfgDescr_TypeDef descrCfg;
  DMA_CfgChannel_TypeDef chnlCfg;

  /* Configure general DMA issues */
  dmaInit.hprot = 0;
  dmaInit.controlBlock = dmaControlBlock;
  DMA_Init(&dmaInit);

  /* Configure DMA channel used */
  chnlCfg.highPri = false;
  chnlCfg.enableInt = false;
  chnlCfg.select = DMAREQ_ADC0_SCAN;
  chnlCfg.cb = NULL;
  DMA_CfgChannel(ACCEL_DMA_CHANNEL, &chnlCfg);

  descrCfg.dstInc = dmaDataInc4;
  descrCfg.srcInc = dmaDataIncNone;
  descrCfg.size = dmaDataSize4;
  descrCfg.arbRate = dmaArbitrate1;
  descrCfg.hprot = 0;
  DMA_CfgDescr(ACCEL_DMA_CHANNEL, true, &descrCfg);
  
  /* Clear all pending DMA interrupts */
  DMA->IFC = _DMA_IFC_MASK;
  DMA->IEN = DMA_IEN_CH0DONE;
  
  /* Set priority for DMA Interrupt */
  NVIC_SetPriority(DMA_IRQn,6);
	
  /* Enable interrupt in DMA Cortex Core */
  NVIC_ClearPendingIRQ(DMA_IRQn);
  NVIC_EnableIRQ(DMA_IRQn);
}


void DMAInitAccel( void )
{
  accelDMAConfig();
}


static void leuartDMAConfig(void)
{
  DMA_Init_TypeDef dmaInit;
  DMA_CfgDescr_TypeDef descrCfg;
  DMA_CfgChannel_TypeDef chnlCfg;

  /* Configure general DMA issues */
  dmaInit.hprot = 0;
  dmaInit.controlBlock = dmaControlBlock;
  DMA_Init(&dmaInit);

  /* Configure DMA channel used */
  chnlCfg.highPri = false;
  chnlCfg.enableInt = false;
  chnlCfg.select = DMAREQ_LEUART1_TXBL;
  chnlCfg.cb = NULL;
  DMA_CfgChannel(LEUART_DMA_CHANNEL, &chnlCfg);

  descrCfg.dstInc = dmaDataIncNone;
  descrCfg.srcInc = dmaDataInc1;
  descrCfg.size = dmaDataSize1;
  descrCfg.arbRate = dmaArbitrate1;
  descrCfg.hprot = 0;  
  DMA_CfgDescr(LEUART_DMA_CHANNEL, true, &descrCfg);
  
    /* Set new DMA destination address directly in the DMA descriptor */
  dmaControlBlock->DSTEND = &LEUART1->TXDATA;

  /* Enable LEUART1 interrupt vector */
  NVIC_EnableIRQ(LEUART1_IRQn);
  
  /* Clear all pending DMA interrupts */
  DMA->IFC = _DMA_IFC_MASK;
  DMA->IEN = DMA_IEN_CH1DONE;
  
  /* Set priority for DMA Interrupt */  
  NVIC_SetPriority(DMA_IRQn,6);
	
  /* Enable interrupt in DMA Cortex Core */
  NVIC_EnableIRQ(DMA_IRQn);
}


void DMAInitLeuart( void )
{
  leuartDMAConfig();
}

void DMA_IRQHandler(void){
  
  portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

  if ( DMA->IF & DMA_IF_CH0DONE){
    
      /* Clear DMA Interrupt register */
      DMA->IFC = DMA_IF_CH0DONE;
      
      /* Set DMA configuration for LEUART */
      DMAInitLeuart();
      
      /* Make a string to send by means of LEUART */
      sprintf(string,"Values sampled from  Accelerometer X= [%d] Y=[%d] Z=[%d] \r\n",samples[0],samples[1],samples[2]);
      
      /* Starting the transfer between Memory and LEUART by means of DMA */
      DMA_ActivateBasic(LEUART_DMA_CHANNEL,  
                    true,                     
                    false,                 
                    (void *)((uint32_t)&(LEUART1->TXDATA)),           
                    string,                       
                    strlen(string));    
  }

  if ( DMA->IF & DMA_IF_CH1DONE){
    
    /* Clear DMA Interrupt register */
    DMA->IFC = DMA_IF_CH1DONE;
    
    /* Unlock EM2 locked for DMA in ADC Handler */
    xPMUnlockSleepModeFromISR(2);

  }
  //portEND_SWITCHING_ISR(xHigherPriorityTaskWoken);
  vPMTicklessFromISR(xHigherPriorityTaskWoken);
}