#include "FreeRTOS.h"

#include "efm32_dma.h"
#include "efm32_cmu.h"
#include "efm32_gpio.h"
#include "efm32_adc.h"

#include "dma.h"
#include "dmactrl.h"
#include "gpio.h"
#include "dvk.h"

#define GPIO_INT_PIN 14


portBASE_TYPE samples[ACCEL_SAMPLES];

/**************************************************************************//**
 * @brief Initialize GPIO interrupt on PC14
 *****************************************************************************/
void GPIO_EVEN_IRQHandler(void)
{
  uint16_t data, button;

  /* Clear interrupt */
  data = DVK_getInterruptFlags();
  DVK_clearInterruptFlags(data);

  /* Clear GPIO interrupt */
  GPIO_IntClear(1 << GPIO_INT_PIN);

  /* Read button status */
  button = DVK_getPushButtons();

  if ( button == BC_PUSHBUTTON_SW1 ){
      
      /* Init DMA for accelerometer */
      DMAInitAccel();   
      
      /* Enable transfer data when ADC comepletes the scan operation */
      DMA_ActivateBasic(ACCEL_DMA_CHANNEL, true, false, samples,
                       (void *)((uint32_t)&(ADC0->SCANDATA)),ACCEL_SAMPLES - 1);
      
      /* Enable ADC scan operation for X, Y, Z */
      ADC_Start(ADC0, adcStartScan);
      
      /* I need to use ADC, therefore I need to stay in EM1 */
      xPMLockSleepModeFromISR(1);
    
  }

}


/**************************************************************************//**
 * @brief Initialize GPIO interrupt on PC14
 *****************************************************************************/
void GPIO_IRQInit(void)
{
  /* Configure interrupt pin as input with pull-up */
  GPIO_PinModeSet(gpioPortC, GPIO_INT_PIN, gpioModeInputPull, 1);

  /* Set falling edge interrupt and clear/enable it */
  GPIO_IntConfig(gpioPortC, GPIO_INT_PIN, false, true, true);

  NVIC_SetPriority(GPIO_EVEN_IRQn,6);
  
  NVIC_ClearPendingIRQ(GPIO_EVEN_IRQn);
  NVIC_EnableIRQ(GPIO_EVEN_IRQn);
}

