#include "FreeRTOS.h"
#include "semphr.h"
#include "efm32_cmu.h"
#include "efm32_leuart.h"
#include "efm32_gpio.h"

/**************************************************************************//**
 * @brief  Initialize Low Energy UART 1
 *
 *
 *****************************************************************************/
void initLeuart(void)
{

  /* Defining the LEUART1 initialization data */
  LEUART_Init_TypeDef leuart1Init;

    /* Start LFXO, and use LFXO for low-energy modules */
  CMU_ClockSelectSet(cmuClock_LFA, cmuSelect_LFXO);
  CMU_ClockSelectSet(cmuClock_LFB, cmuSelect_LFXO);

  /* Enabling clocks, all other remain disabled */
  CMU_ClockEnable(cmuClock_CORELE, true);     /* Enable CORELE clock */
  CMU_ClockEnable(cmuClock_GPIO, true);       /* Enable GPIO clock */
  CMU_ClockEnable(cmuClock_LEUART1, true);    /* Enable LEUART1 clock */
  
  /* Reseting and initializing LEUART1 */
  LEUART_Reset(LEUART1);

  leuart1Init.enable   = leuartEnableTx,       /* Activate data reception on LEUn_TX pin. */
  leuart1Init.refFreq  = 0,                    /* Inherit the clock frequenzy from the LEUART clock source */
  leuart1Init.baudrate = 9600,                 /* Baudrate = 9600 bps */
  leuart1Init.databits = leuartDatabits8,      /* Each LEUART frame containes 8 databits */
  leuart1Init.parity   = leuartNoParity,       /* No parity bits in use */
  leuart1Init.stopbits = leuartStopbits2,      /* Setting the number of stop bits in a frame to 2 bitperiods */

  LEUART_Init(LEUART1, &leuart1Init);

  /* Route LEUART1 TX pin to DMA location 0 */
  LEUART1->ROUTE = LEUART_ROUTE_TXPEN | LEUART_ROUTE_LOCATION_LOC0;

  /* Enable GPIO for LEUART1. TX is on C6 */
  GPIO_PinModeSet(gpioPortC,                /* GPIO port */
                  6,                        /* GPIO port number */
                  gpioModePushPull,         /* Pin mode is set to push pull */
                  1);                       /* High idle state */
				  
  /* Enable LEUART Signal Frame Interrupt */
  LEUART_IntEnable(LEUART1, LEUART_IEN_TXC);

  /* Enable LEUART1 interrupt vector */
  NVIC_EnableIRQ(LEUART1_IRQn);
}

/**************************************************************************//**
* @brief  LEUART IRQ handler
*
* When the DMA transfer is completed, this routine simply clears all the
* LEUART1 interrupt flags. In this way the TXBL flag that is the signal for
* the DMA to be active is cleared, and the DMA is deactivated for energy
* conservation.
*
******************************************************************************/
void LEUART1_IRQHandler(void)
{
  /* Reset all pending interupts */
  LEUART_IntClear(LEUART1, ~_LEUART_IFC_RESETVALUE);
}
