#ifndef PM_H

#include "FreeRTOS.h"
#include "projdefs.h"


#if ( configTICKLESS == 1 )

	/* Tickless Framework */
	#define RTC_FREQ    				32768
	#define PRESC						1
	#define MinTICK						2
	#define MAXRTCOUNT					16777215
	#define TICKRUNNING					0
	#define TICKLESS					1
	#define	TICKLESSPREEMPTED			2
	#define	TICKLESSNOPREEMPTED			3	
#endif

#if ( configUSE_POWERMANAGER == 1 )

	/*Number of possible sleep modes*/
	#define configSLEEPMODES             5

	/* This value is the default sleep mode used by power manager in idle task. If 
	 * the value is 0, then the Power Manager uses as default mode the running state. 
	 * Usually, this value must be set up to the next sleep mode after running state.
	 */
	#define configDEFAULT_SLEEPMODE		1
	 
	/* PAY ATTENTION!!! 
	The order of sleep modes must be ascending and the running mode must always be 0*/
	#define RUNNING			0
	#define EM1				1
	#define EM2				2
	#define EM3				3
	#define EM4				4

	/* ----------- Functions used by users to deal with sleep modes  -------------*/   
	void vPMInit ( void );
	portBASE_TYPE xPMSetSleepMode ( unsigned portCHAR xMode);
	portBASE_TYPE xPMSetSleepModeFromISR ( unsigned portCHAR xMode );
	unsigned portCHAR ucPMGetSleepMode ( void );
	unsigned portCHAR ucPMGetSleepModeFromISR (void);
	portBASE_TYPE xPMLockSleepModeFromISR ( unsigned portCHAR xMode);
	portBASE_TYPE xPMUnlockSleepModeFromISR (unsigned portCHAR xMode);
	portBASE_TYPE xPMUnlockSleepMode (unsigned portCHAR xMode);

	/* Internal function used inside by Power Manager to deal with sleep modes and Tickless */
	void vPMIdleSleepMode ( void ) PRIVILEGED_FUNCTION;
	 
	/* Functions used internally by FreeRTOS to allow a communication between GPMF and Tasks */
	void vPMUpdateSleepModesInc ( unsigned portCHAR xMode );
	portBASE_TYPE xPMUpdateSleepModesDec ( unsigned portCHAR xMode );
	
#endif  
	
#if ( configTICKLESS == 1 )
	/* Tickless mode function */
	void vPMTicklessFromISR( portBASE_TYPE xHigherPriorityTaskWoken);	
#endif
        
 
#endif